@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +18,7 @@
 
 #include <algorithm>
 #include <cerrno>
+#include <climits>
 #include <cstdio>
 #include <cstdlib>
 #include <string>
@@ -35,9 +37,6 @@
 
 namespace {
 
-const std::string unnamed( "(Unnamed)" );
-
-
 std::vector< Point * > & get_marks_and_pointers( Buffer & b )
   {
   static std::vector< Point * > v;
@@ -98,18 +97,19 @@ bool Buffer::Change_history::Record::append_record( const Basic_buffer & buffer,
   }
 
 
-void Buffer::Change_history::Record::delete_atoms() throw()
+void Buffer::Change_history::Record::delete_atoms()
   {
-  for( unsigned int i = 0; i < atom_vector.size(); ++i )
-    if( atom_vector[i].bbp )
-      { delete atom_vector[i].bbp; atom_vector[i].bbp = 0; }
+  for( unsigned i = atom_vector.size(); i > 0; )
+    if( atom_vector[--i].bbp )
+      delete atom_vector[i].bbp;
+  atom_vector.clear();
   }
 
 
-Buffer::Change_history::~Change_history() throw()
+Buffer::Change_history::~Change_history()
   {
-  for( unsigned int i = 0; i < record_vector.size(); ++i )
-    record_vector[i].delete_atoms();
+  for( unsigned i = record_vector.size(); i > 0; )
+    record_vector[--i].delete_atoms();
   }
 
 
@@ -118,8 +118,8 @@ void Buffer::Change_history::add_record( const Basic_buffer & buffer,
   {
   if( index + 1 < records() )
     {
-    for( unsigned int i = index + 1; i < record_vector.size(); ++i )
-      record_vector[i].delete_atoms();
+    for( int i = records(); i > index + 1; )
+      record_vector[--i].delete_atoms();
     record_vector.erase( record_vector.begin() + index + 1, record_vector.end() );
     appendable = false;
     }
@@ -137,29 +137,30 @@ void Buffer::Change_history::add_record( const Basic_buffer & buffer,
   }
 
 
-bool Buffer::Options::set_lmargin( const int l ) throw()
+bool Buffer::Options::set_lmargin( const int l )
   {
-  if( l >= 0 && l < _rmargin ) { _lmargin = l; return true; }
+  if( l >= 0 && l < rmargin_ ) { lmargin_ = l; return true; }
   return false;
   }
 
 
-bool Buffer::Options::set_rmargin( const int r ) throw()
+bool Buffer::Options::set_rmargin( const int r )
   {
-  if( r > _lmargin && r < 1000 ) { _rmargin = r; return true; }
+  if( r > lmargin_ && r < 1000 ) { rmargin_ = r; return true; }
   return false;
   }
 
 
-Buffer::Buffer( const Options & opts, const std::string * namep )
-  : _modified( false ), saved( false ), options( opts ), crlf( false )
+Buffer::Buffer( const Options & opts, const std::string * const namep )
+  : backuped( false ), modified_( false ), saved( false ),
+    options( opts ), crlf( false )
   {
   FILE *f = 0;
   errno = 0;
   if( ( !namep || !namep->size() || *namep == "-" ) && !isatty( fileno( stdin ) ) )
     f = stdin;
   else if( namep && namep->size() )
-    { _name = *namep; f = std::fopen( _name.c_str(), "r" ); }
+    { name_ = *namep; f = std::fopen( name_.c_str(), "r" ); }
   else return;
 
   if( !f )
@@ -187,7 +188,7 @@ Buffer::Buffer( const Options & opts, const std::string * namep )
   if( file_eof ) file_eof = std::feof( f );
   if( f != stdin )
     {
-    if( std::freopen( _name.c_str(), "r+", f ) ) std::fclose( f );
+    if( std::freopen( name_.c_str(), "r+", f ) ) std::fclose( f );
     else options.read_only = true;
     }
   else if( !ttyname( 1 ) || !std::freopen( ttyname( 1 ), "r", stdin ) )
@@ -195,29 +196,35 @@ Buffer::Buffer( const Options & opts, const std::string * namep )
   if( file_error ) throw Error( "Error reading file" );
   if( !file_eof ) throw Error( "Not enough memory" );
   crlf = detect_and_remove_cr();
-  if( lines() >= 23 ) mark[0].line = 22;
+  for( int i = 10; i < 60 && i < lines(); ++i )
+    if( !get_line( i ).compare( 0, 10, "#include \"" ) )
+      { mark[0].line = i - 1; break; }
+  if( mark[0].line < 0 )
+    for( int i = 19; i < 30 && i < lines(); ++i )
+      if( blank( i ) ) { mark[0].line = i; break; }
+  if( mark[0].line < 0 ) mark[0].line = 22;
   for( int i = 1; i < 10 && lines() >= 100 * i; ++i )
     mark[i].line = 100 * i - 1;
   }
 
 
-bool Buffer::save( const std::string * namep, bool set_name, bool append,
-                   Point p1, Point p2 ) const
+bool Buffer::save( const std::string * namep, bool set_name,
+                   bool append, Point p1, Point p2 ) const
   {
   int mode = 0;
   struct stat st;
   if( append ) mode = S_IRUSR | S_IWUSR;
-  else if( ( namep && namep->size() && !stat( namep->c_str(), &st ) ) ||
-           ( _name.size() && !stat( _name.c_str(), &st ) ) )
+  else if( ( namep && namep->size() && stat( namep->c_str(), &st ) == 0 ) ||
+           ( name_.size() && stat( name_.c_str(), &st ) == 0 ) )
          mode = st.st_mode;
 
-  const bool had_name = ( _name.size() > 0 );
+  const bool had_name = ( name_.size() > 0 );
   bool same_name = set_name;
   if( set_name )
     {
     if( namep && namep->size() )
       {
-      if( _name != *namep )
+      if( name_ != *namep )
         {
         FILE *f = std::fopen( namep->c_str(), "r+" );
         if( f )
@@ -227,29 +234,30 @@ bool Buffer::save( const std::string * namep, bool set_name, bool append,
           if( key != 'y' && key != 'Y' ) return false;
           }
         }
-      if( !_name.size() ) _name = *namep;
-      else if( _name != *namep ) same_name = false;
+      if( !name_.size() ) name_ = *namep;
+      else if( name_ != *namep ) same_name = false;
       }
-    else { if( !_name.size() ) return false; else namep = &_name; }
+    else { if( !name_.size() ) return false; else namep = &name_; }
     }
-  if( !namep ) return false;
+  if( !namep || ( same_name && options.read_only ) ) return false;
 
-  if( same_name )
+  if( !Menu::make_path( *namep ) )
+    { if( !had_name ) { name_.clear(); } throw Error( "Error creating path" ); }
+  if( same_name && !saved && !append )
     {
-    if( options.read_only ) return false;
-    if( !saved && RC::editor_options().backup )
+    if( RC::editor_options().backup && !backuped )
       {
       std::string backname( *namep ); backname += '~';
       std::remove( backname.c_str() );
       std::rename( namep->c_str(), backname.c_str() );
+      backuped = true;
       }
+    else std::remove( namep->c_str() );
     }
 
-  FILE *f = 0;
-  if( Menu::make_path( *namep ) )
-    f = std::fopen( namep->c_str(), append ? "a" : "w" );
+  FILE *f = std::fopen( namep->c_str(), append ? "a" : "w" );
   if( !f )
-    { if( !had_name ) { _name.clear(); } throw Error( "Error opening file" ); }
+    { if( !had_name ) { name_.clear(); } throw Error( "Error opening file" ); }
   if( mode ) fchmod( fileno( f ), mode );
 
   if( !this->pisvalid( p1 ) ) p1 = this->bof();
@@ -262,51 +270,101 @@ bool Buffer::save( const std::string * namep, bool set_name, bool append,
     if( ch == '\n' ) std::fputc( '\r', f );
     std::fputc( ch, f );
     }
-  std::fclose( f );
+  if( std::fclose( f ) != 0 )
+    { if( !had_name ) { name_.clear(); } throw Error( "Error closing file" ); }
   if( same_name )
     {
     for( int i = change_history.records() - 1; i >= 0; --i )
       if( change_history.record_vector[i].modified == false )
         { change_history.record_vector[i].modified = true; break; }
     reset_appendable();
-    _modified = false; saved = true;
+    modified_ = false; saved = true;
     }
   return true;
   }
 
 
-const std::string & Buffer::character_info( const Point & p ) const throw()
+long Buffer::calculate_offset( const Point & p, int & percent ) const
   {
-  static std::string s;
-  char tmp1[60], tmp2[40];
-  int ch = (*this)[p];
+  long total = 0;
 
-  if( ch >= 0 )
-    snprintf( tmp2, sizeof tmp2, "Code %d(0x%02X/0%o) **", ch, ch ,ch );
-  else if( p == eof() )
-    snprintf( tmp2, sizeof tmp2, "(End Of File) **" );
-  else
-    { snprintf( tmp2, sizeof tmp2, "*** Error: Bad Pointer: Line %d  Col %d",
-                p.line + 1, p.col + 1 ); s = tmp2; return s; }
-
-  int percent = 0;
-  for( int i = 0; i < p.line; ++i ) percent += characters( i );
-  int offset = percent + p.col;
-  if( crlf ) { offset += p.line; if( ch == '\n' ) ++offset; }
-  for( int i = p.line; i < eof().line; ++i ) percent += characters( i );
-  if( crlf ) percent += eof().line;
-  percent += eof().col;
-  if( percent > 0 ) percent = ( 100LL * offset ) / percent;
-
-  snprintf( tmp1, sizeof tmp1, "** Line %d  Col %d  Offset %d(0x%X)  %d%%  ",
-            p.line + 1, p.col + 1, offset, offset, percent );
-  s = tmp1; s += tmp2;
-  return s;
+  for( int i = 0; i < p.line; ++i ) total += characters( i );
+  long offset = total + p.col;
+  if( crlf ) { offset += p.line; if( (*this)[p] == '\n' ) ++offset; }
+  for( int i = p.line; i < last_line(); ++i ) total += characters( i );
+  if( crlf ) total += last_line();
+  total += eof().col;
+  if( total > 0 ) percent = ( 100LL * offset ) / total;
+  else percent = 0;
+  return offset;
   }
 
 
-const std::string & Buffer::uname() const throw()
-  { if( _name.size() ) return _name; else return unnamed; }
+void Buffer::character_info( const Point & p, std::string & s ) const
+  {
+  char buf1[64], buf2[64];
+  const int ch = (*this)[p];
+
+  if( ch >= 0 )
+    snprintf( buf2, sizeof buf2, "Code %d(0x%02X/0%o)", ch, ch ,ch );
+  else if( p == eof() )
+    snprintf( buf2, sizeof buf2, "(End Of File)" );
+  else
+    { snprintf( buf2, sizeof buf2, "*** Error: Bad Pointer: Line %d  Pos %d",
+                p.line + 1, p.col + 1 ); s = buf2; return; }
+
+  int percent = 0;
+  long offset = calculate_offset( p, percent );
+
+  snprintf( buf1, sizeof buf1, "Line %d  Pos %d  Offset %ld(0x%lX)  %d%%  ",
+            p.line + 1, p.col + 1, offset, offset, percent );
+  s = buf1; s += buf2;
+  }
+
+
+void Buffer::multichar_value( const Point & p, std::string & s,
+                              const int size, const bool big_endian ) const
+  {
+  int bufsize = 0;
+  char * const buf = Screen::line_buf( &bufsize );
+  if( size < 2 || size > 8 )
+    {
+    snprintf( buf, bufsize, "*** Error: Bad Multichar Size: %d", size );
+    s = buf; return;
+    }
+  std::string text;
+  const int sz = to_string( p, size, text );
+  int percent = 0;
+  long offset = calculate_offset( p, percent );
+
+  if( sz >= size )
+    {
+    long long value = 0;
+    if( big_endian )
+      for( int i = 0; i < size; ++i )
+        { value <<= 8; value += (unsigned char)text[i]; }
+    else
+      for( int i = size - 1; i >= 0; --i )
+        { value <<= 8; value += (unsigned char)text[i]; }
+    snprintf( buf, bufsize, "Off %ld(0x%lX)  %s_en_%d %lld(0x%0*llX)",
+              offset, offset, big_endian ? "Big" : "Lit",
+              size, value, 2 * size, value );
+    }
+  else if( sz > 0 )
+    snprintf( buf, bufsize, "Offset %ld(0x%lX)  %d%%  (Only %d Characters Left)",
+              offset, offset, percent, sz );
+  else
+    snprintf( buf, bufsize, "Offset %ld(0x%lX)  %d%%  (End Of File)",
+              offset, offset, percent );
+  s = buf;
+  }
+
+
+const std::string & Buffer::nename() const
+  {
+  static const std::string unnamed( "(Unnamed)" );
+  if( name_.size() ) return name_; else return unnamed;
+  }
 
 
 // insert block b.[p1,p2) at p and move next
@@ -320,15 +378,15 @@ bool Buffer::pputb( Point & p, const Basic_buffer & b, const Point & p1,
   if( save_change )		// save_change == 0 only for undo/redo
     {
     const Point & after = (save_change != 2) ? p0 : p;
-    Change_history::Record r( p0, after, _modified );
+    Change_history::Record r( p0, after, modified_ );
     r.add_atom( Change_history::Atom( p0, p ) );
     change_history.add_record( *this, r );
-    _modified = true;
+    modified_ = true;
     }
 
   const int line_dif = p2.line - p1.line;
   std::vector< Point * > & pv = get_marks_and_pointers( *this );
-  for( unsigned int i = 0; i < pv.size(); ++i )
+  for( unsigned i = 0; i < pv.size(); ++i )
     if( *pv[i] > p0 && pv[i] != &p )
       {
       if( pv[i]->line == p0.line )
@@ -373,7 +431,7 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
     if( options.overwrite && p < eol( p ) )
       {
       const Point p1 = Point( p.line, p.col + 1 );
-      Change_history::Record r( p, p1, _modified );
+      Change_history::Record r( p, p1, modified_ );
       Change_history::Atom a( p, p1, *this );
       Basic_buffer::pchgc( p, ch );
       a.end = p1; r.add_atom( a );
@@ -383,11 +441,11 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
       {
       const Point p0 = p;
       Basic_buffer::pputc( p, ch );
-      Change_history::Record r( p0, p, _modified );
+      Change_history::Record r( p0, p, modified_ );
       r.add_atom( Change_history::Atom( p0, p ) );
       change_history.add_record( *this, r );
       std::vector< Point * > & pv = get_marks_and_pointers( *this );
-      for( unsigned int i = 0; i < pv.size(); ++i )
+      for( unsigned i = 0; i < pv.size(); ++i )
         if( pv[i]->line == p0.line && pv[i]->col > p0.col && pv[i] != &p )
           ++pv[i]->col;
       if( options.word_wrap && to_cursor( p ).col > options.rmargin() + 1 &&
@@ -410,7 +468,7 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
           }
         }
       }
-    _modified = true;
+    modified_ = true;
     }
   return true;
   }
@@ -424,16 +482,16 @@ bool Buffer::pdelb( const Point & p1, const Point & p2, const int save_change )
   if( save_change )		// save_change == 0 only for undo/redo
     {
     const Point & p0 = (save_change != 2) ? p1 : p2;
-    Change_history::Record r( p0, p1, _modified );
+    Change_history::Record r( p0, p1, modified_ );
     r.add_atom( Change_history::Atom( p1, p2, *this ) );
     change_history.add_record( *this, r );
-    _modified = true;
+    modified_ = true;
     }
   Basic_buffer::pdelb( p1, p2 );
 
   const int line_dif = p2.line - p1.line;
   std::vector< Point * > & pv = get_marks_and_pointers( *this );
-  for( unsigned int i = 0; i < pv.size(); ++i )
+  for( unsigned i = 0; i < pv.size(); ++i )
     if( *pv[i] > p1 )
       {
       if( *pv[i] <= p2 ) *pv[i] = p1;
@@ -457,7 +515,7 @@ bool Buffer::pdelc( Point & p, const bool back )
 
 
 // delete line and set 'p' to bol()
-bool Buffer::pdell( Point & p ) throw()
+bool Buffer::pdell( Point & p )
   {
   Point p1 = eol( p );
   if( options.read_only || characters( p.line ) <= 0 ||
@@ -471,7 +529,7 @@ bool Buffer::pdell( Point & p ) throw()
 bool Buffer::replace( const Point & p1, Point & p2, const Basic_buffer & b,
                       const Point & bp1, const Point & bp2 )
   {
-  Change_history::Record r( p1, p1, _modified );
+  Change_history::Record r( p1, p1, modified_ );
   Change_history::Atom a( p1, p2, *this );
   if( !pdelb( p1, p2, 0 ) && a.bbp ) { delete a.bbp; a.bbp = 0; }
   p2 = p1;
@@ -479,7 +537,7 @@ bool Buffer::replace( const Point & p1, Point & p2, const Basic_buffer & b,
   if( a.begin < a.end || a.bbp )
     {
     r.add_atom( a ); change_history.add_record( *this, r );
-    _modified = true;
+    modified_ = true;
     return true;
     }
   return false;
@@ -490,9 +548,9 @@ bool Buffer::undo( Point & p )
   {
   if( change_history.index < 0 ) return false;
   Change_history::Record & rec = change_history.record_vector[change_history.index];
-  for( unsigned int i = 0; i < rec.atom_vector.size(); ++i )
+  for( unsigned i = rec.atom_vector.size(); i > 0; )
     {
-    Change_history::Atom & atom = rec.atom_vector[rec.atom_vector.size()-i-1];
+    Change_history::Atom & atom = rec.atom_vector[--i];
     Basic_buffer *tmp = 0;
     if( atom.begin < atom.end )
       {
@@ -508,7 +566,7 @@ bool Buffer::undo( Point & p )
     atom.bbp = tmp;
     }
   p = rec.before;
-  std::swap( _modified, rec.modified );
+  std::swap( modified_, rec.modified );
   --change_history.index;
   return true;
   }
@@ -519,7 +577,7 @@ bool Buffer::redo( Point & p )
   if( change_history.index + 1 >= change_history.records() ) return false;
   ++change_history.index;
   Change_history::Record & rec = change_history.record_vector[change_history.index];
-  for( unsigned int i = 0; i < rec.atom_vector.size(); ++i )
+  for( unsigned i = 0; i < rec.atom_vector.size(); ++i )
     {
     Change_history::Atom & atom = rec.atom_vector[i];
     Basic_buffer *tmp = 0;
@@ -537,6 +595,6 @@ bool Buffer::redo( Point & p )
     atom.bbp = tmp;
     }
   p = rec.after;
-  std::swap( _modified, rec.modified );
+  std::swap( modified_, rec.modified );
   return true;
   }
