@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,11 +23,19 @@ enum Keys { key_up, key_down, key_left, key_right, key_npage, key_ppage,
 
 bool init();
 
-//void beep() throw();
+int height();
+int width();
+int max_windows();
 
-const std::string & clock_string() throw();
+// return pointer to an array width() + 1 bytes in size
+//
+char * line_buf( int * const sizep );
 
-extern "C" void clock_handler( int sig ) throw();	// Updates the clock.
+//void beep();
+
+const char * clock_string();
+
+extern "C" void clock_handler( int sig );	// Updates the clock.
 
 int kbhit();			// Returns -1 if no key is available.
 
@@ -45,30 +54,26 @@ int wait_kbhit( const int new_clock_limit = -1, const bool get_key = true,
 //
 int get_string( const std::string & prompt,
                 std::vector< std::string > & history,
-                const int abort_key = 3,
-                const bool allow_copy = false, const bool usetab = false );
+                const int abort_key = 3, const bool usetab = false );
 
 
-int height() throw();
-int width() throw();
-int max_windows() throw();
-
-void move_to( const Point & cursor ) throw();
-void save_cursor( Point & cursor ) throw();
+void move_to( const Point & cursor );
+void save_cursor( Point & cursor );
 void save_lines_and_cursor( const int line, const int lines );
-void restore_lines_and_cursor() throw();
+void restore_lines_and_cursor();
 
-void out_buffer_line( const Buffer & buffer, Point c, const int line ) throw();
+void out_buffer_line( const Buffer & buffer, Point c, const int line );
 bool out_line( const std::string & s, const int line, const bool shift = false,
-               const char mode = 0 ) throw();
+               const char mode = 0 );
 bool out_menu_line( const std::string & s, const int line,
-                    const int hbegin = -1, const int hend = -1 ) throw();
-void repaint( const Buffer * bufp = 0 ) throw();
+                    const int hbegin = -1, const int hend = -1 );
+void repaint( const Buffer * const bufp = 0 );
 int show_message( const std::string & s, const bool get_key = false,
                   const bool take_cursor = false );
-void show_status_lines( const Buffer * bufp = 0 ) throw();
+void show_status_lines( const Buffer * const bufp = 0 );
 
-Keys convert_key( const int key ) throw();
-int user_loop( const std::string & version_msg ) throw();
+Keys convert_key( const int key );
+int function_key( const int key );		// return F number or -1
+int user_loop( const std::string & version_msg );
 
 } // end namespace Screen
