@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
     Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
@@ -32,7 +32,7 @@
 #include "window.h"
 
 
-Window::Window( Buffer_handle & bh, const int tl, const int h, const bool center ) throw()
+Window::Window( Buffer_handle & bh, const int tl, const int h, const bool center )
   : bhp( &bh ), top_line_( tl ), height_( h )
   {
   bhp->buffer().pvalid( bhp->pointer );
@@ -41,33 +41,33 @@ Window::Window( Buffer_handle & bh, const int tl, const int h, const bool center
   }
 
 
-Point Window::clock_position() const throw()
+Point Window::clock_position() const
   { return Point( top_line_, Screen::width() - 20 ); }
 
 
-Point Window::absolute_cursor() const throw()	// cursor position on screen
-  { return relative_cursor() + Point( top_line_, 0 ); }
+Point Window::relative_cursor() const	// cursor position on window
+  { return bhp->cursor - bhp->top_left; }
 
 
-Point Window::relative_cursor() const throw()	// cursor position on window
-  { return Point( 1, 0 ) + bhp->cursor - bhp->top_left; }
+Point Window::absolute_cursor() const	// cursor position on screen
+  { return relative_cursor() + Point( top_line_ + 1, 0 ); }
 
 
-void Window::center_cursor() throw()
+void Window::center_cursor()
   {
   bhp->top_left.line = bhp->buffer().lines();
   update_points( bhp->pointer, true, true );
   }
 
 
-void Window::goto_bof() throw()
+void Window::goto_bof()
   {
   Point bof = bhp->buffer().bof();
   if( bhp->pointer != bof ) update_points( bof );
   }
 
 
-void Window::goto_eof() throw()
+void Window::goto_eof()
   {
   Point eof = bhp->buffer().eof();
   if( bhp->pointer != eof || bhp->cursor != bhp->pcursor )
@@ -75,7 +75,7 @@ void Window::goto_eof() throw()
   }
 
 
-void Window::goto_home() throw()
+void Window::goto_home()
   {
   Point p = bhp->pointer;
   const Buffer & buffer = bhp->buffer();
@@ -90,7 +90,7 @@ void Window::goto_home() throw()
   }
 
 
-void Window::goto_eol() throw()
+void Window::goto_eol()
   {
   Point eol = bhp->buffer().eol( bhp->pointer );
   if( bhp->cursor != bhp->pcursor || bhp->pointer != eol )
@@ -98,7 +98,7 @@ void Window::goto_eol() throw()
   }
 
 
-void Window::goto_begin_of_block() throw()
+void Window::goto_begin_of_block()
   {
   if( Block::bufferp() == &buffer() && buffer().pisvalid( Block::begin() ) )
     update_points( Block::begin(), true, true );
@@ -108,7 +108,7 @@ void Window::goto_begin_of_block() throw()
   }
 
 
-void Window::goto_end_of_block() throw()
+void Window::goto_end_of_block()
   {
   if( Block::bufferp() == &buffer() && buffer().pisvalid( Block::end() ) )
     update_points( Block::end(), true, true );
@@ -126,13 +126,14 @@ void Window::goto_line()
   if( Screen::get_string( "Go to line (^C to abort): ", history ) <= 0 )
     return;
   const std::string & s = history.back();
-  Point p( 0, 0 );
-  if( !RC::parse_int( s, p.line ) || p.line <= 0 )
+  Point c = bhp->cursor;
+  ++c.line;
+  if( !RC::parse_relative_int( s, c.line ) || c.line <= 0 )
     { Screen::show_message( "Invalid line number" ); history.pop_back();
-    return; }
-  --p.line;
-  if( !bhp->buffer().pisvalid( p ) ) p = bhp->buffer().eof();
-  update_points( p, true, true );
+      return; }
+  --c.line;
+  if( c.line > bhp->buffer().last_line() ) c.line = bhp->buffer().last_line();
+  update_points( c, true, true, from_cursor );
   }
 
 
@@ -145,9 +146,10 @@ void Window::goto_column()
     return;
   const std::string & s = history.back();
   Point c = bhp->cursor;
-  if( !RC::parse_int( s, c.col ) || c.col <= 0 )
+  ++c.col;
+  if( !RC::parse_relative_int( s, c.col ) || c.col <= 0 )
     { Screen::show_message( "Invalid column number" ); history.pop_back();
-    return; }
+      return; }
   --c.col;
   update_points( c, true, false, from_cursor );
   }
@@ -161,23 +163,40 @@ void Window::goto_offset()
   if( Screen::get_string( "Go to offset (^C to abort): ", history ) <= 0 )
     return;
   const std::string & s = history.back();
-  Point p( 0, 0 );
-  if( !RC::parse_int( s, p.col ) || p.col < 0 )
+  const bool relative = RC::issigned( s );
+  int offset = 0;
+  if( !RC::parse_int( s, offset ) || ( !relative && offset < 0 ) )
     { Screen::show_message( "Invalid offset value" ); history.pop_back();
-    return; }
+      return; }
   const int cr = buffer().crlf;
-  for( ; p.line < buffer().last_line(); ++p.line )
+  Point p( 0, 0 );
+  if( relative ) p = bhp->pointer;
+  p.col += offset;
+  if( offset > 0 )
     {
-    const int c = buffer().characters( p.line );
-    if( p.col < c + cr ) { p.col = std::min( p.col, c - 1 ); break; }
-    p.col -= ( c + cr );
+    while( p.line < buffer().last_line() )
+      {
+      const int c = buffer().characters( p.line );
+      if( p.col < c + cr ) { p.col = std::min( p.col, c - 1 ); break; }
+      p.col -= ( c + cr ); ++p.line;
+      }
+    if( !bhp->buffer().pisvalid( p ) ) p = bhp->buffer().eof();
+    }
+  else if( offset < 0 && p.col < 0 )
+    {
+    while( p.line > 0 )
+      {
+      const int c = buffer().characters( --p.line );
+      p.col += c + cr;
+      if( p.col >= 0 ) { p.col = std::min( p.col, c - 1 ); break; }
+      }
+    if( !bhp->buffer().pisvalid( p ) ) p = bhp->buffer().bof();
     }
-  if( !bhp->buffer().pisvalid( p ) ) p = bhp->buffer().eof();
   update_points( p, true, true );
   }
 
 
-void Window::goto_mark( const int i ) throw()
+void Window::goto_mark( const int i )
   {
   if( i >= 0 && i <= 9 )
     {
@@ -194,7 +213,7 @@ void Window::goto_mark( const int i ) throw()
   }
 
 
-void Window::goto_matching_delimiter( const bool forward ) throw()
+void Window::goto_matching_delimiter( const bool forward )
   {
   Point p = bhp->pointer;
   const Buffer & buffer = bhp->buffer();
@@ -223,7 +242,7 @@ void Window::goto_matching_delimiter( const bool forward ) throw()
   }
 
 
-void Window::goto_pnext() throw()
+void Window::goto_pnext()
   {
   Point p = bhp->pointer;
   const Buffer & buffer = bhp->buffer();
@@ -232,7 +251,7 @@ void Window::goto_pnext() throw()
   }
 
 
-void Window::goto_pprev() throw()
+void Window::goto_pprev()
   {
   Point p = bhp->pointer;
   const Buffer & buffer = bhp->buffer();
@@ -241,19 +260,21 @@ void Window::goto_pprev() throw()
   }
 
 
-void Window::move_page( const bool down ) throw()
+void Window::move_page( const bool down, const bool scroll )
   {
-  const int keep_lines = RC::editor_options().keep_lines;
-  int lines = ( height_ - 1 ) / 2;
+  const int keep_lines =
+    std::min( RC::editor_options().keep_lines(), height_ - 2 );
+  int lines = height_ - 1;
 
-  if( keep_lines >= 0 && keep_lines <= lines )
-    lines = height_ - 1 - keep_lines;
+  if( keep_lines >= 0 ) lines -= keep_lines;
+  else lines /= 2;
   if( !down ) lines = -lines;
-  move_vertical( lines );
+  if( !scroll ) move_vertical( lines );
+  else scroll_vertical( lines );
   }
 
 
-void Window::move_vertical( const int lines ) throw()
+void Window::move_vertical( const int lines )
   {
   Point c = bhp->cursor;
 
@@ -261,7 +282,7 @@ void Window::move_vertical( const int lines ) throw()
     { if( c.line == 0 ) { return; } c.line += lines; c.cut(); }
   else if( lines > 0 )
     {
-    int last_line = bhp->buffer().last_line();
+    const int last_line = bhp->buffer().last_line();
     if( c.line == last_line ) return;
     c.line += lines; if( c.line > last_line ) c.line = last_line;
     }
@@ -271,7 +292,7 @@ void Window::move_vertical( const int lines ) throw()
   }
 
 
-void Window::scroll_horizontal( const int cols ) throw()
+void Window::scroll_horizontal( const int cols )
   {
   Point tl = bhp->top_left;
 
@@ -283,23 +304,12 @@ void Window::scroll_horizontal( const int cols ) throw()
   }
 
 
-void Window::scroll_page( const bool down ) throw()
-  {
-  const int keep_lines = RC::editor_options().keep_lines;
-  int lines = ( height_ - 1 ) / 2;
-
-  if( keep_lines >= 0 && keep_lines < lines )
-    lines = height_ - 1 - keep_lines;
-  if( !down ) lines = -lines;
-  scroll_vertical( lines );
-  }
-
-
-void Window::scroll_vertical( const int lines ) throw()
+void Window::scroll_vertical( const int lines )
   {
   Point tl = bhp->top_left;
 
-  if( lines < 0 ) { if( tl.line == 0 ) { return; } tl.line += lines; tl.cut(); }
+  if( lines < 0 )
+    { if( tl.line == 0 ) { return; } tl.line += lines; tl.cut(); }
   else if( lines > 0 )
     {
     int first_line = bhp->buffer().last_line() - ( height_ - 2 );
@@ -312,7 +322,7 @@ void Window::scroll_vertical( const int lines ) throw()
   }
 
 
-void Window::set_mark( const int i ) throw()
+void Window::set_mark( const int i )
   {
   if( i >= 0 && i <= 9 )
     {
@@ -324,7 +334,7 @@ void Window::set_mark( const int i ) throw()
   }
 
 
-void Window::repaint() const throw()
+void Window::repaint() const
   {
   show_status_line();
   for( int i = 0; i < height_ - 1; ++i )
@@ -333,7 +343,7 @@ void Window::repaint() const throw()
   }
 
 
-void Window::show_character_info() const throw()
+void Window::show_character_info() const
   {
   std::string s;
   bhp->buffer().character_info( bhp->pointer, s );
@@ -341,9 +351,17 @@ void Window::show_character_info() const throw()
   }
 
 
+void Window::show_multichar_value( const int size, const bool big_endian ) const
+  {
+  std::string s;
+  bhp->buffer().multichar_value( bhp->pointer, s, size, big_endian );
+  Screen::show_message( s );
+  }
+
+
 // prefix == keyboard status ( ^K, ^Q, ^[, etc )
 //
-void Window::show_status_line( const char * const prefix ) const throw()
+void Window::show_status_line( const char * const prefix ) const
   {
   if( top_line_ < 0 ) return;
   int size = 0;
@@ -367,11 +385,11 @@ void Window::show_status_line( const char * const prefix ) const throw()
                             modified ? " (Modified)" : "" );
   if( len < offset ) std::memset( buf + len, ' ', offset - len );
   snprintf( buf + offset, size - offset,
-            "  Line %-4d Col %-3d %5s  F1 for help",
+            "  Line %-4d Col %-3d %s  F1 for help",
             line, col, Screen::clock_string() );
   if( RC::editor_options().show_code )
     {
-    offset = size - 14;
+    offset = size - 13;
     const int ch = bhp->buffer()[bhp->pointer];
     if( ch >= 0 )
       snprintf( buf + offset, size - offset, "%3d(0x%02X)", ch, ch );
@@ -383,9 +401,9 @@ void Window::show_status_line( const char * const prefix ) const throw()
 
 
 void Window::update_points( const Point & to, const bool show,
-                    bool center, const From from ) throw()
+                            bool center, const From from )
   {
-  Buffer_handle & bh = buffer_handle();
+  Buffer_handle & bh = *bhp;
   const Buffer & buffer = bh.buffer();
   Point & tl = bh.top_left;
   Point & c = bh.cursor;
@@ -402,23 +420,28 @@ void Window::update_points( const Point & to, const bool show,
       center = false;
       break;
       }
-    case from_cursor: c = to; c.cut(), p = buffer.to_pointer( c, pc ); break;
+    case from_cursor: c = to; c.cut(); p = buffer.to_pointer( c, pc ); break;
     case from_pointer: p = to; c = pc = buffer.to_cursor( p ); break;
     }
 
   const Point rc = relative_cursor();
-  if( rc.line > height_ - 2 )
+  if( rc.line > height_ - 3 )
     {
-    const int d = center ? ( height_ - 2 ) / 2 : height_ - 2;
+    const int d = ( center ? ( height_ - 2 ) / 2 :
+                    height_ - ( ( c.line >= buffer.last_line() ) ? 2 : 3 ) );
     tl.line = std::max( 0, c.line - d );
     }
   else if( rc.line < 1 )
     {
-    const int d = center ? ( height_ - 2 ) / 2 : 0;
+    const int d = ( center ? ( height_ - 2 ) / 2 : 1 );
     tl.line = std::max( 0, c.line - d );
     }
-  if( rc.col >= Screen::width() || rc.col <= 0 || center )
-    tl.col = std::max( 0, c.col - Screen::width() + 1 );
+  const int ahead = std::max( 1, std::min( 8,
+                                 buffer.characters( p.line ) - p.col ) );
+  if( rc.col + ahead > Screen::width() || center )
+    tl.col = std::max( 0, c.col + ahead - Screen::width() );
+  else if( rc.col < 8 )
+    tl.col = std::max( 0, c.col - 8 );
 
   if( show ) { if( tl == old_tl ) show_status_line(); else repaint(); }
   }
