@@ -1,5 +1,5 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
     Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
@@ -42,14 +42,14 @@ int curwin_;			// current window (the one with the cursor in)
 std::vector< Window > data;
 
 
-bool no_block() throw()
+bool no_block()
   {
   if( !Block::valid() ) { Screen::show_message( "No block" ); return true; }
   return false;
   }
 
 
-bool no_block_in_this_file( const Buffer & buffer ) throw()
+bool no_block_in_this_file( const Buffer & buffer )
   {
   if( Block::bufferp() != &buffer )
     { Screen::show_message( "Block is not in this file" ); return true; }
@@ -57,7 +57,7 @@ bool no_block_in_this_file( const Buffer & buffer ) throw()
   }
 
 
-bool rectangular_block() throw()
+bool rectangular_block()
   {
   if( RC::editor_options().rectangle_mode )
     {
@@ -68,7 +68,7 @@ bool rectangular_block() throw()
   }
 
 
-bool read_only( const Buffer & buffer ) throw()
+bool read_only( const Buffer & buffer )
   {
   if( buffer.options.read_only )
     { Screen::show_message( "Read only" ); return true; }
@@ -76,7 +76,7 @@ bool read_only( const Buffer & buffer ) throw()
   }
 
 
-void replace_and_repaint( Buffer & buffer, const Basic_buffer & tmp ) throw()
+void replace_and_repaint( Buffer & buffer, const Basic_buffer & tmp )
   {
   Point p = Block::end();
   buffer.reset_appendable();
@@ -111,13 +111,13 @@ int Window_vector::load( int new_size )
   else if( new_size < 0 )
     {
     for( int i = 0; i < curwin_; ++i ) Bufhandle_vector::next();
-    new_size = (data.size() != 1) ? 1 : handles;
+    new_size = ( ( data.size() != 1 ) ? 1 : handles );
     curwin_ = 0;
     }
   new_size = std::min( new_size, handles );
   new_size = std::min( new_size, Screen::max_windows() );
-  if( RC::editor_options().max_windows > 0 )
-    new_size = std::min( new_size, RC::editor_options().max_windows );
+  if( RC::editor_options().max_windows() > 0 )
+    new_size = std::min( new_size, RC::editor_options().max_windows() );
   new_size = std::max( new_size, 1 );
   while( curwin_ >= new_size ) { --curwin_; Bufhandle_vector::next(); }
   const int wheight = Screen::height() / new_size;
@@ -146,10 +146,18 @@ void Window_vector::bufhandle_menu( const int abort_key )
 void Window_vector::change_buffer_name()
   {
   Buffer & buffer = data[curwin_].buffer();
-
   std::string name( buffer.name() );
-  if( Menu::get_filename( "New buffer name (^C to abort): ", name ) < 0 )
+
+  if( Menu::get_filename( "New buffer name (^C to abort): ", name ) < 0 ||
+      name == buffer.name() )
     return;
+  for( int i = 0; i < Bufhandle_vector::handles(); ++i )
+    if( name == Bufhandle_vector::handle( i ).buffer().name() &&
+        &buffer != &Bufhandle_vector::handle( i ).buffer() )
+      {
+      std::string s( "File '" ); s += name; s += "' is already being edited";
+      Screen::show_message( s ); return;
+      }
   buffer.name( name );
   Screen::show_status_lines( &buffer );
   }
@@ -158,12 +166,13 @@ void Window_vector::change_buffer_name()
 void Window_vector::load_file( const int abort_key )
   {
   std::string name;
-  if( Menu::get_filename( "Name of file to edit (^C to abort): ", name, abort_key ) < 0 )
+  if( Menu::get_filename( "Name of file to edit (^C to abort): ",
+                          name, abort_key ) < 0 )
     return;
   const int old_handles = Bufhandle_vector::handles();
   int handle;
-  try { handle = Bufhandle_vector::find_or_add_handle( RC::default_buffer_options(),
-                                                       &name, curwin_ + 1 ); }
+  try { handle = Bufhandle_vector::find_or_add_handle(
+                 RC::default_buffer_options(), &name, curwin_ + 1 ); }
   catch( Buffer::Error e ) { Screen::show_message( e.msg ); return; }
   curwin_ = handle;
   if( windows() > 1 && Bufhandle_vector::handles() > old_handles )
@@ -203,8 +212,7 @@ void Window_vector::save_file( const int abort_key )
 //
 int Window_vector::close( bool abort, const bool quiet )
   {
-  Window & w = data[curwin_];
-  Buffer_handle & bh = w.buffer_handle();
+  Buffer_handle & bh = data[curwin_].buffer_handle();
   const bool last_handle = Bufhandle_vector::handles( bh.buffer() ) <= 1;
   const bool modified = bh.buffer().modified();
   const bool exit_ask = RC::editor_options().exit_ask && !quiet;
@@ -254,13 +262,12 @@ int Window_vector::close_and_exit( const bool abort )
   }
 
 
-Window & Window_vector::curwin() throw() { return data[curwin_]; }
+Window & Window_vector::curwin() { return data[curwin_]; }
 
 
 void Window_vector::last_visited()
   {
-  const int handles = Bufhandle_vector::handles();
-  if( handles <= 1 ) return;
+  if( Bufhandle_vector::handles() <= 1 ) return;
   curwin_ = Bufhandle_vector::last_visited( curwin_ );
   if( curwin_ >= windows() ) load();
   }
@@ -292,17 +299,17 @@ void Window_vector::prev()
   }
 
 
-Window & Window_vector::window( int i ) throw()
+Window & Window_vector::window( int i )
   {
   if( i < 0 ) i = 0; else if( i >= windows() ) i = windows() - 1;
   return data[i];
   }
 
 
-int Window_vector::windows() throw() { return data.size(); }
+int Window_vector::windows() { return data.size(); }
 
 
-void Window_vector::add_char( const int key, const bool force ) throw()
+void Window_vector::add_char( const int key, const bool force )
   {
   Window & w = data[curwin_];
   if( w.buffer().options.read_only )
@@ -325,7 +332,7 @@ void Window_vector::add_char( const int key, const bool force ) throw()
   }
 
 
-void Window_vector::delete_char( const bool back ) throw()
+void Window_vector::delete_char( const bool back )
   {
   Window & w = data[curwin_];
   Point & p = w.pointer();
@@ -335,7 +342,7 @@ void Window_vector::delete_char( const bool back ) throw()
   }
 
 
-void Window_vector::delete_line() throw()
+void Window_vector::delete_line()
   {
   Window & w = data[curwin_];
 
@@ -344,7 +351,7 @@ void Window_vector::delete_line() throw()
   }
 
 
-void Window_vector::copy_block() throw()
+void Window_vector::copy_block()
   {
   Window & w = data[curwin_];
 
@@ -360,7 +367,7 @@ void Window_vector::copy_block() throw()
   }
 
 
-void Window_vector::delete_block() throw()
+void Window_vector::delete_block()
   {
   Buffer & buffer = data[curwin_].buffer();
 
@@ -370,7 +377,7 @@ void Window_vector::delete_block() throw()
   }
 
 
-void Window_vector::move_block() throw()
+void Window_vector::move_block()
   {
   Window & w = data[curwin_];
   Point & p = w.pointer();
@@ -435,7 +442,7 @@ void Window_vector::write_block()
   }
 
 
-void Window_vector::indent_block() throw()
+void Window_vector::indent_block()
   {
   Buffer & buffer = data[curwin_].buffer();
 
@@ -452,7 +459,7 @@ void Window_vector::indent_block() throw()
     if( p.col > 0 || p < buffer.eol( p ) )
       {
       if( first_post ) first_post = false; else buffer.force_append();
-      for( int i = 0; i < RC::editor_options().indent_step; ++i )
+      for( int i = 0; i < RC::editor_options().indent_step(); ++i )
         buffer.pputc( p, ' ' );
       }
     ++p.line;
@@ -464,7 +471,7 @@ void Window_vector::indent_block() throw()
   }
 
 
-void Window_vector::unindent_block() throw()
+void Window_vector::unindent_block()
   {
   Buffer & buffer = data[curwin_].buffer();
 
@@ -475,7 +482,7 @@ void Window_vector::unindent_block() throw()
   while( buffer.bol( p ) < Block::end() && !fail )
     {
     if( buffer.eol( p ).col > 0 &&
-        buffer.to_cursor( buffer.bot( p ) ).col < RC::editor_options().indent_step )
+        buffer.to_cursor( buffer.bot( p ) ).col < RC::editor_options().indent_step() )
       fail = true;
     ++p.line;
     }
@@ -488,7 +495,8 @@ void Window_vector::unindent_block() throw()
   p = Block::begin();
   while( buffer.bol( p ) < Block::end() )
     {
-    const int target_col = buffer.to_cursor( buffer.bot( p ) ).col - RC::editor_options().indent_step;
+    const int target_col = buffer.to_cursor( buffer.bot( p ) ).col -
+                           RC::editor_options().indent_step();
     if( buffer.eol( p ).col > 0 )
       while( true )
         {
@@ -515,7 +523,7 @@ void Window_vector::unindent_block() throw()
   }
 
 
-void Window_vector::undo() throw()
+void Window_vector::undo()
   {
   Window & w = data[curwin_];
 
@@ -527,7 +535,7 @@ void Window_vector::undo() throw()
   }
 
 
-void Window_vector::redo() throw()
+void Window_vector::redo()
   {
   Window & w = data[curwin_];
 
@@ -539,7 +547,7 @@ void Window_vector::redo() throw()
   }
 
 
-void Window_vector::center_line() throw()
+void Window_vector::center_line()
   {
   Window & w = data[curwin_];
   Buffer & buffer = w.buffer();
@@ -567,23 +575,28 @@ void Window_vector::center_line() throw()
   }
 
 
-void Window_vector::reformat() throw()
+void Window_vector::reformat()
   {
   Window & w = data[curwin_];
-  Buffer & buffer = w.buffer();
-  const Point bop = buffer.bop( w.pointer() );
-  const Point eop = buffer.eop( w.pointer() );
+  Buffer_handle & bh = w.buffer_handle();
+  Buffer & buffer = bh.buffer();
+  const Point bop = buffer.bop( bh.pointer );
+  const Point eop = buffer.eop( bh.pointer );
   const Point bot = buffer.bot( bop );
 
   if( !read_only( buffer ) && bop < eop )
     {
+    const int lmargin = std::max( buffer.options.lmargin(),
+                                  std::min( buffer.options.rmargin() - 1,
+                                            bh.cursor.col ) );
     Buffer tmp( buffer.options );
     tmp.options.auto_indent = false;
     tmp.options.overwrite = false;
     tmp.options.word_wrap = true;
     Point dummy = tmp.bof();
-    if( bop < bot && buffer.to_cursor( bot ).col > buffer.options.lmargin() )
+    if( bop < bot && buffer.to_cursor( bot ).col > lmargin )
       tmp.Basic_buffer::pputb( dummy, buffer, bop, bot );
+    tmp.options.set_lmargin( lmargin );
     bool space = false;
     for( Point p = bot; p < eop; )
       {
@@ -609,12 +622,14 @@ void Window_vector::reformat() throw()
       buffer.replace( bop, p, tmp, tmp.bof(), tmp.eof() );
       buffer.reset_appendable();
       Screen::repaint( &buffer );
+      p.line = bop.line; p.col = lmargin;
+      w.update_points( p, true, false, Window::from_cursor );
       }
     }
   }
 
 
-void Window_vector::change_case( const bool to_caps ) throw()
+void Window_vector::change_case( const bool to_caps )
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || read_only( buffer ) )
@@ -645,7 +660,7 @@ void Window_vector::change_case( const bool to_caps ) throw()
   }
 
 
-void Window_vector::encode_base64() throw()
+void Window_vector::encode_base64()
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
@@ -656,7 +671,7 @@ void Window_vector::encode_base64() throw()
   Basic_buffer tmp;
   Point p = tmp.bof();
   int c = 0;
-  for( unsigned int i = 0; i < out.size(); ++i )
+  for( unsigned i = 0; i < out.size(); ++i )
     {
     tmp.pputc( p, out[i] );
     if( ++c >= 72 ) { c = 0; tmp.pputc( p, '\n' ); }
@@ -666,7 +681,7 @@ void Window_vector::encode_base64() throw()
   }
 
 
-void Window_vector::decode_base64() throw()
+void Window_vector::decode_base64()
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
@@ -680,13 +695,13 @@ void Window_vector::decode_base64() throw()
     { Screen::show_message( "Invalid Base64 data in block" ); return; }
   Basic_buffer tmp;
   Point p = tmp.bof();
-  for( unsigned int i = 0; i < out.size(); ++i )
+  for( unsigned i = 0; i < out.size(); ++i )
     tmp.pputc( p, out[i] );
   replace_and_repaint( buffer, tmp );
   }
 
 
-void Window_vector::decode_quoted_printable_utf8( const bool mode_q ) throw()
+void Window_vector::decode_quoted_printable_utf8( const bool mode_q )
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
@@ -703,13 +718,13 @@ void Window_vector::decode_quoted_printable_utf8( const bool mode_q ) throw()
     { Screen::show_message( "Text in block is already decoded" ); return; }
   Basic_buffer tmp;
   Point p = tmp.bof();
-  for( unsigned int i = 0; i < out.size(); ++i )
+  for( unsigned i = 0; i < out.size(); ++i )
     tmp.pputc( p, out[i] );
   replace_and_repaint( buffer, tmp );
   }
 
 
-void Window_vector::encode_rot1347( const bool mode13 ) throw()
+void Window_vector::encode_rot1347( const bool mode13 )
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
@@ -726,7 +741,7 @@ void Window_vector::encode_rot1347( const bool mode13 ) throw()
   }
 
 
-void Window_vector::remove_duplicate_lines( const bool back ) throw()
+void Window_vector::remove_duplicate_lines( const bool back )
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
@@ -791,7 +806,7 @@ const char * Window_vector::copyright_update()
   if( Screen::get_string( "Year to add (^C to abort): ", history ) <= 0 )
     return 0;
   year_string = history.back();
-  for( unsigned int i = 0; i < year_string.size(); ++i )
+  for( unsigned i = 0; i < year_string.size(); ++i )
     if( !std::isdigit( year_string[i] ) )
       year_string.clear();
   if( !year_string.size() || year_string[0] == '0' ) return "Invalid year";
@@ -828,7 +843,7 @@ const char * Window_vector::copyright_update()
           tmp.pputb( pt, w.buffer(), p0, p1 );
           }
         else tmp.pputc( pt, ' ' );
-        for( unsigned int i = 2; i < year_string.size() - 2; ++i )
+        for( unsigned i = 2; i < year_string.size() - 2; ++i )
           tmp.pputc( pt, year_string[i] );
         if( ph < p2 )
           {
@@ -896,7 +911,10 @@ const char * Window_vector::search( const int abort_key, const bool again,
   static std::vector< std::string > options_history;
   static std::string search_string;
   static std::string replace_string;
-  static bool backward = false, block = false, global = false, icase = false;
+  static bool backward = false;
+  static bool block = false;
+  static bool global = false;
+  static bool icase = false;
   static bool replace = false;
 
   if( !again || !search_string.size() )
@@ -922,7 +940,7 @@ const char * Window_vector::search( const int abort_key, const bool again,
     if( size > 0 )
       {
       const std::string & options = options_history.back();
-      for( unsigned int i = 0; i < options.size(); ++i )
+      for( unsigned i = 0; i < options.size(); ++i )
         switch( ISO_8859::toupper( options[i] ) )
           {
           case 'B': backward = true; break;
@@ -933,6 +951,7 @@ const char * Window_vector::search( const int abort_key, const bool again,
           case 'R': replace = true; break;
           }
       }
+    if( block ) global = false;
     if( replace )
       {
       replace_string.clear();
@@ -947,7 +966,7 @@ const char * Window_vector::search( const int abort_key, const bool again,
     }
   if( toggle ) backward = !backward;
 
-  Point p = curwin().pointer();
+  Point p = curwin().pointer();			// search point
   if( block )
     {
     if( no_block() || no_block_in_this_file( curwin().buffer() ) ) return 0;
@@ -956,7 +975,8 @@ const char * Window_vector::search( const int abort_key, const bool again,
     }
   bool found = false, ask = true, found_any = false, wrapped = false;
   bool empty_replace = false;
-  for( int i = (global ? Bufhandle_vector::handles() - 1 : 0); i >= 0 && !found; --i )
+  const int handles = ( global ? Bufhandle_vector::handles() : 1 );
+  for( int i = handles - 1; i >= 0 && !found; --i )
     {
     Window & w = data[curwin_];
     std::vector< std::string > regex_pieces;
